﻿using System.Diagnostics;
using DerpingDrivers.Exceptions;
using Microsoft.Win32;

namespace DerpingDrivers.Util
{
    public static class BcdHelper
    {
        public static bool IsTestSignEnabled
        {
            get
            {
                var bootmgrDefaultGuid = (string) Registry.GetValue(
                    @"HKEY_LOCAL_MACHINE\BCD00000000\Objects\{9DEA862C-5CDD-4E70-ACC1-F32B344D4795}\Elements\23000003",
                    "Element", null);

                var allowPrereleaseSignatures = (byte[]) Registry.GetValue(
                    $@"HKEY_LOCAL_MACHINE\BCD00000000\Objects\{bootmgrDefaultGuid}\Elements\16000049",
                    "Element", default(byte[]));

                return allowPrereleaseSignatures != null && int.Parse(allowPrereleaseSignatures[0].ToString()) > 0;
            }
        }

        public static void EnableTestSigning()
        {
            // NOTE: haven't found an API for this (yet), so system() we go!
            var cmd = new Process
            {
                StartInfo =
                {
                    FileName = "Bcdedit.exe",
                    Arguments = "-set TESTSIGNING ON",
                    WindowStyle = ProcessWindowStyle.Hidden,
                    CreateNoWindow = true
                }
            };

            cmd.Start();
            cmd.WaitForExit();

            if (cmd.ExitCode != 0)
                throw new BcdAlterAllowPrereleaseSignaturesFailedException("Couldn't enable TESTSIGNING");
        }

        public static void DisableTestSigning()
        {
            // NOTE: haven't found an API for this (yet), so system() we go!
            var cmd = new Process
            {
                StartInfo =
                {
                    FileName = "Bcdedit.exe",
                    Arguments = "-set TESTSIGNING OFF",
                    WindowStyle = ProcessWindowStyle.Hidden,
                    CreateNoWindow = true
                }
            };

            cmd.Start();
            cmd.WaitForExit();

            if (cmd.ExitCode != 0)
                throw new BcdAlterAllowPrereleaseSignaturesFailedException("Couldn't disable TESTSIGNING");
        }
    }
}